@@ -1,27 +1,21 @@
 from gettext import gettext as _
-import re, os, cgi
+import re, os
 import gobject
 import gnomevfs
-import deskbar.Handler
-import deskbar.Match
+import deskbar.interfaces.Module
+import deskbar.interfaces.Match
 from deskbar.defs import VERSION
-from deskbar.Utils import spawn_async, url_show
+from deskbar.core.Utils import spawn_async, url_show
 
-HANDLERS = {
-	"WebAddressHandler" : {
-		"name": _("Web"),
-		"description": _("Open web pages and send emails by typing a complete address"),
-		"version": VERSION,
-	}
-}
+HANDLERS = ["WebAddressHandler"]
 
 AUTH_REGEX = re.compile(r'[a-zA-Z]+://\w+(:\w+)?@([\w\-]+\.)+[\w\-]+(:\d+)?(/.*)?')
 HTTP_REGEX = re.compile(r'^(?P<method>[a-zA-Z]+://)?([\w\-]+\.)+[\w\-]+(:\d+)?(/.*)?$')
 MAIL_REGEX = re.compile(r'^([\w\-]+\.)*[\w\-]+@([\w\-]+\.)*[\w\-]+$')
 
-class WebAddressMatch(deskbar.Match.Match):
-	def __init__(self, backend, name=None, url=None, has_method=True, **args):
-		deskbar.Match.Match.__init__(self, backend, name=name, **args)
+class WebAddressMatch(deskbar.interfaces.Match):
+	def __init__(self, name=None, url=None, has_method=True, **args):
+		deskbar.interfaces.Match.__init__(self, name=name, icon="stock_internet", category="web", **args)
 		self.url = url
 		
 		self.has_method = has_method
@@ -33,9 +27,6 @@ class WebAddressMatch(deskbar.Match.Match):
 			url_show(self.url)
 		else:
 			spawn_async(["nautilus", self.url])
-			
-	def get_category(self):
-		return "web"
 	
 	def get_verb(self):
 		if not self.has_method:
@@ -46,16 +37,13 @@ class WebAddressMatch(deskbar.Match.Match):
 	def get_hash(self, text=None):
 		return self.url
 
-class EmailAddressMatch(deskbar.Match.Match):
-	def __init__(self, backend, name=None, mail=None, **args):
-		deskbar.Match.Match.__init__(self, backend, name=name, icon="stock_mail", **args)
+class EmailAddressMatch(deskbar.interfaces.Match):
+	def __init__(self, name=None, mail=None, **args):
+		deskbar.interfaces.Match.__init__(self, name=name, icon="stock_mail", category="people", **args)
 		self.mail = mail
 		
 	def action(self, text=None):
 		url_show("mailto:"+self.mail)
-		
-	def get_category(self):
-		return "people"
 	
 	def get_verb(self):
 		return _("Send Email to %s") % "<b>%(name)s</b>"
@@ -63,28 +51,34 @@ class EmailAddressMatch(deskbar.Match.Match):
 	def get_hash(self, text=None):
 		return self.mail
 		
-class WebAddressHandler(deskbar.Handler.Handler):
+class WebAddressHandler(deskbar.interfaces.Module):
+	
+	INFOS = {'icon': deskbar.core.Utils.load_icon("stock_internet"),
+			 "name": _("Web"),
+			 "description": _("Open web pages and send emails by typing a complete address"),
+			 "version": VERSION}
+	
 	def __init__(self):
-		deskbar.Handler.Handler.__init__(self, "stock_internet")
+		deskbar.interfaces.Module.__init__(self)
 	
 	def query(self, query):
 		result = self.query_http(query)
 		result += self.query_mail(query)
-		return result
+		self._emit_query_ready(query, result )
 		
 	def query_http(self, query):
 		match = AUTH_REGEX.match(query)
 		if match != None:
-			return [WebAddressMatch(self, query)]
+			return [WebAddressMatch(query)]
 		
 		match = HTTP_REGEX.match(query)
 		if match != None:
-			return [WebAddressMatch(self, cgi.escape(query), query, (match.group('method') != None))]
+			return [WebAddressMatch(query, query, (match.group('method') != None), priority=self.get_priority())]
 	
 		return []
 		
 	def query_mail(self, query):
 		if MAIL_REGEX.match(query) != None:
-			return [EmailAddressMatch(self, cgi.escape(query), query)]
+			return [EmailAddressMatch(query, query, priority=self.get_priority())]
 		else:
 			return []
