@@ -1,10 +1,10 @@
 import gtk
+import gtk.gdk
 import gobject
-
 import cgi
-
 import deskbar
-from deskbar.Categories import CATEGORIES
+import deskbar.interfaces.Match
+from deskbar.core.Categories import CATEGORIES
 from deskbar.ui.cuemiac.CuemiacItems import CuemiacCategory
 			
 # The sort function ids
@@ -41,55 +41,51 @@ class CuemiacModel (gtk.TreeStore):
 		self.__categories = {}
 		self.append_method = gtk.TreeStore.append # Alternatively gtk.TreeStore.prepend for bottom panel layout
 		self.set_sort_func(SORT_BY_CATEGORY, self.__on_sort_categories)
+		self.set_sort_order(gtk.SORT_DESCENDING)
 	
 	def set_sort_order(self, order):
 		self.set_sort_column_id(SORT_BY_CATEGORY, order)
 	
-	def __on_sort_categories(self, treemodel, iter1, iter2):
-		match1 = treemodel[iter1][self.MATCHES]
-		match2 = treemodel[iter2][self.MATCHES]
-
-		# Sort categories according to handler preferences
-		if match1.__class__ == CuemiacCategory:
-			return match1.get_priority() - match2.get_priority()
-				
-		# Sort matches inside category according to handler prefs, then match prio
-		text1, match1 = match1
-		text2, match2 = match2
-		
-		diff = match1.get_priority()[0] - match2.get_priority()[0]
-		if diff != 0:
-			return diff
-			
-		diff = match1.get_priority()[1] - match2.get_priority()[1]
-		if diff != 0:
-			return diff
-		
-		# Finally use the Action to sort alphabetically, is this a bottleneck ?
-		a = treemodel[iter1][self.ACTIONS]
-		b = treemodel[iter2][self.ACTIONS]
-		if a != None:
-			a = a.strip().lower()
-		if b != None:
-			b = b.strip().lower()
-			
-		if a < b:
+	def __compare(self, item1, item2):
+		if (item1 > item2):
 			return 1
-		elif a > b:
+		elif (item1 < item2):
 			return -1
 		else:
 			return 0
+	
+	def __on_sort_categories(self, treemodel, iter1, iter2):
+		match1 = treemodel[iter1][self.MATCHES]
+		match2 = treemodel[iter2][self.MATCHES]
 		
-	def append (self, match):
+		if isinstance(match1, CuemiacCategory):
+			return self.__compare( match1.get_priority(), match2.get_priority() )
+		
+		text1, match_obj1 = match1
+		text2, match_obj2 = match2
+		if match_obj1.get_priority() == match_obj2.get_priority():
+			# Sort alphabetically
+			a = treemodel[iter1][self.ACTIONS]
+			b = treemodel[iter2][self.ACTIONS]
+			if a != None and b != None:
+				return self.__compare( a.strip().lower(), b.strip().lower() )
+		
+		return self.__compare( match_obj1.get_priority(), match_obj2.get_priority() )
+	
+	def append (self, match_obj, query_string):
 		"""
 		Automagically append a match or list of matches 
 		to correct category(s), or create a new one(s) if needed.
 		"""
-		if type (match) == list:
-			for hit in match:
-				self.__append (hit)
+		#gtk.gdk.threads_enter()
+		if isinstance(match_obj, list):
+			for hit in match_obj:
+				self.__append ( (query_string, hit) )
+		elif isinstance(match_obj, deskbar.interfaces.Match):
+			self.__append ( (query_string, match_obj) )
 		else:
-			self.__append (match)
+			raise RuntimeError("Unknown Match type")
+		#gtk.gdk.threads_leave()
 		
 	def __append (self, match):
 		qstring, match_obj = match
@@ -106,6 +102,7 @@ class CuemiacModel (gtk.TreeStore):
 		qstring, match_obj = match
 		#FIXME: Check validity of category name and use  proper i18n
 		# Set up a new category
+		
 		cat = CuemiacCategory (match_obj.get_category(), self)
 		cat.set_priority(match_obj)	
 
@@ -117,7 +114,6 @@ class CuemiacModel (gtk.TreeStore):
 		self.__append_match_to_iter (iter, match)
 		cat.inc_count ()
 		self.emit ("category-added", cat, cat.get_category_row_path ())
-		
 	
 	def __append_to_category (self, match):
 		qstring, match_obj = match
@@ -129,13 +125,13 @@ class CuemiacModel (gtk.TreeStore):
 		self.__append_match_to_iter (cat.get_category_iter(), match)
 				
 		# Update the row count in the view:
-		self.row_changed (cat.get_category_row_path(), cat.get_category_iter())
+		self.row_changed (cat.get_category_row_path(), cat.get_category_iter())		
 
 	def __append_match_to_iter (self, iter, match):
 		qstring, match_obj = match
 		# Pass unescaped query to the matches
 		verbs = {"text" : qstring}
-		verbs.update(match_obj.get_name(qstring))
+		verbs.update(match_obj.get_escaped_name(qstring))
 		# Escape the query now for display
 		verbs["text"] = cgi.escape(verbs["text"])
 		
